setGeneric("retrieve",
           function(from, to, ...) standardGeneric("retrieve"))

setClass("FileReferences",
         representation=representation(
           urls="character",
           localNames="character",
           types="character"))

setValidity("FileReferences", function(object)
{
    msg <- NULL

    furls <- urls(object)
    lnames <- localNames(object)
    types <- types(object)
    if (length(furls) != length(lnames) || length(furls) != length(types)) {
        txt <- sprintf("%s must have same length",
                       "'urls', 'localNames', 'types'")
        msg <- c(msg, txt)
    }

    if (any(is.na(furls)) || any(is.na(lnames)) || any(is.na(types))) {
        txt <- sprintf("%s must not have NA elements",
                       "'urls', 'localNames', 'types'")
        msg <- c(msg, txt)
    }
        
    if (is.null(msg)) TRUE else msg
})

urls <- function(object, ...) {
    slot(object, "urls")
}

localNames <- function(object, ...) {
    slot(object, "localNames")
}

types <- function(object, ...) {
    slot(object, "types")
}

setMethod(length, "FileReferences", function(x) {
    length(urls(x))
})

FileReferences <-
    function(urls=character(), localNames=basename(urls),
             types=character(length(urls)), ...)
{
    new("FileReferences",
        urls=urls, localNames=localNames, types=types, ...)
}

setMethod(retrieve, c("FileReferences", "missing"),
          function(from, to, ...) callGeneric(from, to=tempfile(), ...))

setMethod(retrieve, c("FileReferences", "character"),
          function(from, to, ...) 
{
    if (length(to) != 1)
        stop("'to' must be 'character(1)'")
    if (!file.exists(to))
        dir.create(to, recursive=TRUE)
    else if (!file.info(to)$isdir)
        stop("'to' must be a directory")
    localUrls <- unlist(lapply(urls(from), function(from, toDir) {
        toFile <- tempfile(tmpdir=toDir)
        writeBin(getBinaryURL(from, ...), toFile)
        toFile
    }, toDir=to))
    initialize(from, urls=localUrls)
})

setMethod(retrieve, c("FileReferences", "FileReferences"),
          function(from, to, ...) 
{
    fromUrls <- urls(from)
    toUrls <- urls(to)
    if (length(fromUrls) != length(toUrls))
        stop("'from' and 'to' urls must have same length")
    if (any(sapply(toUrls, file.exists)))
        stop("'to' url(s) must not already exist")
    for (i in seq_along(fromUrls)) {
        writeBin(getBinaryURL(fromUrls[[i]], ...), toUrls[[i]])
    }
    initialize(from, urls=toUrls)
})


## The following are added to regAddonCvt and regAddonCvt2 in
## basicConvert.R and basicConvert2.R. They also need to be referenced
## in mkDataMap

cvtFileReferencesToJava <- function(x, ...)
    .JNew("org.bioconductor.packages.rservices.RFileReferences",
          urls(x), localNames(x), types(x), .convert=FALSE)
matchFileReferencesToJava <- function(x, ...)
    is(x, "FileReferences")
cvtFileReferencesFromJava <- function(x, thisClassName)
    FileReferences(urls=x$getUrls(), localNames=x$getLocalNames(),
                   types=x$getTypes())
matchFileReferencesFromJava <- function(x, thisClassName)
    thisClassName == "org.bioconductor.packages.rservices.RFileReferences"

cvtFileReferencesToJava2 <- function(x, ...)
    .JNew("org.bioconductor.packages.rservices.RJFileReferences",
          urls(x), localNames(x), types(x))
matchFileReferencesToJava2 <- function(x, ...)
    is(x, "FileReferences")
cvtFileReferencesFromJava2 <- function(x, thisClassName)
    FileReferences(urls=x$getUrls(), localNames=x$getLocalNames(),
                   types=x$getTypes())
matchFileReferencesFromJava2 <- function(x, thisClassName)
    thisClassName == "org.bioconductor.packages.rservices.RJFileReferences"
