setGeneric("createMap",
           function(funcs, pkgs,
                    generateTests=TRUE,
                    outputDirectory=stop("specify outputDirectory"),
                    typeMode="javalib",
                    deployMode="jms",
                    verbose=FALSE,
                    ...) standardGeneric("createMap"),
           signature=c("funcs", "pkgs"))

asValidNames <- function(nms) {
    ## remove embedded . and replace with camel-case
    nms <- gsub("([a-zA-Z1-9])\\.([a-zA-Z1-9])","\\1\\U\\2", nms, perl=TRUE)
    ## leading .
    nms <- gsub("^\\.", "_", nms)
    nms
}

setMethod("createMap",
          signature(funcs="missing", pkgs="character"),
          function(funcs, pkgs, generateTests=TRUE,
                   outputDirectory=stop("specify outputDirectory"),
                   typeMode="javalib", deployMode="jms",
                   verbose=FALSE, splitPkgsToVector=TRUE, ...,
                   extraClasses=character()) {
              if (splitPkgsToVector && length(pkgs)==1)
                  pkgs <- unlist(strsplit(pkgs, ","))
              sigs <- lapply(as.list(pkgs), function(pkg) {
                  if (!require(pkg, quietly=TRUE, character.only=TRUE))
                    stop("could not find package '", pkg, "'")
                  typed <- unlist(eapply(getNamespace(pkg), function(elt) {
                      "TypeInfo" %in% names(attributes(eval(elt)))
                  }))
                  nms <- asValidNames(names(typed[typed]))
                  mapply(typeInfo2Java,
                         lapply(nms, get, envir=getNamespace(pkg)),
                         nms)
              })
              extraClasses1 <- unlist(lapply(as.list(pkgs), function(pkg) {
                  loc <- system.file("rservices", "properties.R", package=pkg)
                  if (nzchar(loc)) {
                      source(loc, TRUE) # source into calling env, i.e., here
                      if (exists("extraClasses")) extraClasses
                      else character(0)
                  } else character(0)
              }))
              extraClasses <- unique(c(extraClasses, extraClasses1))
              typeMode1 <- unique(unlist(lapply(as.list(pkgs), function(pkg) {
                  loc <- system.file("rservices", "properties.R", package=pkg)
                  if (nzchar(loc)) {
                      source(loc, TRUE)
                      if (exists("typeMode")) typeMode
                      else character(0)
                  } else character(0)
              })))
              
              if (length(typeMode1) == 1 && typeMode1 != typeMode) {
                  warning("using typeMode '", typeMode1, "' from package rservices directory")
                  typeMode <- typeMode1
              } else if (length(typeMode1) > 1)
                  stop("'typeMode' must be identical for all mapped packages")
              generateFunctionMap(unlist(sigs, recursive=FALSE),
                                  genTest=generateTests,
                                  workDir=outputDirectory,
                                  verbose=verbose,
                                  deployModeName=deployMode,
                                  typeModeName=typeMode,
                                  ..., extraClasses=extraClasses)
          })

setMethod("createMap",
          signature(funcs="standardGeneric", pkgs="missing"),
          function(funcs, pkgs,
                   generateTests, outputDirectory, typeMode, deployMode, verbose,
                   ...) {
              sigs <- typeInfo2Java(funcs, asValidNames(deparse(substitute(funcs))), ...)
              generateFunctionMap(sigs,
                                  genTest=generateTests,
                                  workDir=outputDirectory,
                                  verbose=verbose,
                                  deployModeName=deployMode,
                                  typeModeName=typeMode,
                                  ...)
          })

setMethod("createMap",
          signature(funcs="character", pkgs="missing"),
          function(funcs, pkgs,
                   generateTests, outputDirectory, typeMode, deployMode, verbose,
                   ...) {
              nms <- asValidNames(funcs)
              funcs <- lapply(funcs, get, envir=parent.frame())
              sigs <- typeInfo2Java(funcs, nms)
              if (verbose) { cat("typeInfo2Java:\n"); print(sigs) }
              generateFunctionMap(sigs,
                                  genTest=generateTests,
                                  workDir=outputDirectory,
                                  verbose=verbose,
                                  deployModeName=deployMode,
                                  typeModeName=typeMode,
                                  ...)
          })
