/**
 * RJFactor.java
 * 
 * This java class corresponds to R factor 
 */
package org.bioconductor.packages.rservices;

import java.util.Arrays;

public class RJFactor implements java.io.Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected String[] levels = new String[0];

	protected int[] code = new int[0];

	public RJFactor() {
	}

	public RJFactor(String[] levels, int[] code) {
		this.levels = levels;
		this.code = code;
	}

	/**
	 * Sets the levels for this RJFactor.
	 * 
	 * @param levels
	 */
	public void setLevels(String[] levels) {
		this.levels = levels;
	}

	/**
	 * Sets the levels for this RJFactor.
	 * 
	 * @param levels
	 */
	public void setLevels(String levels) {
		this.levels = new String[] { levels };
	}

	/**
	 * Gets the levels for this RArray.
	 * 
	 * @return levels
	 */
	public String[] getLevels() {
		return levels;
	}

	/**
	 * Sets the code for this RJFactor.
	 * 
	 * @param code
	 */
	public void setCode(int[] code) {
		this.code = code;
	}

	/**
	 * Sets the code for this RJFactor.
	 * 
	 * @param code
	 */
	public void setCode(Integer code) {
		this.code = new int[] { code.intValue() };
	}

	/**
	 * Gets the code value for this RJFactor.
	 * 
	 * @return code
	 */
	public int[] getCode() {
		return code;
	}

	/**
	 * Gets the String[] representation of this RJFactor. i.e. all the code are
	 * converted to their corresponding levels
	 * 
	 * @return data
	 */
	public String[] asData() {
		String[] data = new String[code.length];
		for (int i = 0; i < code.length; i++) {
			if (code[i] < 0) {
				data[i] = "NA";
			} else {
				data[i] = levels[code[i]];
			}
		}
		return data;
	}

	@Override
	public boolean equals(Object inputObject) {
		boolean res = getClass().equals(inputObject.getClass());
		if (res) {
			RJFactor obj = (RJFactor) inputObject;
			int[] objCode = obj.getCode();
			String[] objLevels = obj.getLevels();
			res = res && Arrays.equals(code, objCode);
			res = res && Arrays.equals(levels, objLevels);
		}
		return res;
	}

	@Override
	public String toString() {
		StringBuffer res = new StringBuffer("");
		res.append("{ data=[ ");
		for (int element : code) {
			if (element < 0) {
				res.append("NA ");
			} else {
				res.append(levels[element]).append(" ");
			}
		}
		res.append("], levels=[");
		for (String element : levels) {
			res.append(element).append(" ");
		}
		res.append("] }");
		return res.toString();
	}
}
