/**
 * RJNumericArray.java
 * 
 * This java class corresponds to R NumericArray 
 */
package org.bioconductor.packages.rservices;

import java.util.Arrays;

public class RJNumericArray extends RJArray {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected double[] value = new double[0];

	public RJNumericArray() {
	}

	public RJNumericArray(double[] value, int[] dim, Object[] dimnames) {
		this.value = value;
		this.dim = dim;
		this.dimnames = dimnames;
	}

	/**
	 * Sets the value for this RJNumericArray.
	 * 
	 * @param value
	 */
	public void setValue(double[] value) {
		this.value = value;
	}

	/**
	 * Gets the value for this RJNumericArray.
	 * 
	 * @return value
	 */
	public double[] getValue() {
		return value;
	}

	@Override
	public boolean equals(Object inputObject) {
		boolean res = getClass().equals(inputObject.getClass());
		if (res) {
			RJNumericArray obj = (RJNumericArray) inputObject;
			int[] objDim = obj.getDim();
			double[] objValue = obj.getValue();
			Object[] objDimnames = obj.getDimnames();
			res = res && Arrays.equals(value, objValue);
			if ((dim == null) || (objDim == null)) {
				res = res && (dim == objDim);
			} else {
				res = res && Arrays.equals(objDim, dim);
			}
			if ((dimnames == null) || (objDimnames == null)) {
				res = res && (dimnames == objDimnames);
			} else {
				res = res && Arrays.deepEquals(dimnames, objDimnames);
			}
		}
		return res;
	}

	@Override
	public String toString() {
		StringBuffer res = new StringBuffer("RJNumericArray {");
		res.append("value= " + Arrays.toString(value));
		if (dim == null) {
			res.append(", dim=null");
		} else {
			res.append(", dim= " + Arrays.toString(dim));
		}
		if (dimnames == null) {
			res.append(", dimnames=null");
		} else {
			res.append(", dimnames= " + Arrays.deepToString(dimnames));
		}
		res.append(" }");
		return res.toString();
	}
}
