package org.bioconductor.packages.rservices;

/*
 These test the converters between R and Java.
 Specifically, methods below pass simple data
 from Java to R and get the results back. This
 round-trip should test many of the converters.
*/

import java.util.*;
import java.lang.reflect.Array;
import org.omegahat.R.Java.ROmegahatInterpreter;
import org.omegahat.R.Java.REvaluator;
import org.bioconductor.packages.rservices.*;
import junit.framework.JUnit4TestAdapter;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.Ignore;
import static org.junit.Assert.*;

public class RObjectsTest extends org.bioconductor.packages.rservices.TypeTestUtil {

    /**                                                                                          
     * Used for backward compatibility (IDEs, Ant and JUnit 3.x text runner)
     */
    public static junit.framework.Test suite() {
        return new JUnit4TestAdapter(RObjectsTest.class);
    }

    /**
     * Sets up the test fixture; Called before all the tests; Called only once.
     */
    @BeforeClass
    public static void loadRWebServicesConverter() throws Exception {
        e.voidEval("regAddonCvt()");
    }
    
    @Test
    public void testNull() throws Exception {
      assertNull(reflectObj(null));
    }

    @Test
    public void testRRaw() throws Exception {
      RRaw inputVal=new RRaw();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setValue(new byte[]{15, 16, 17});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setNames(new String[]{"n1", "n2", "n3"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRLogical() throws Exception {
      RLogical inputVal=new RLogical();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setValue(new boolean[]{true , false, true});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setIndexNA(new int[]{2});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setNames(new String[]{"n1", "n2", "n3"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRNumeric() throws Exception {
      RNumeric inputVal=new RNumeric();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setValue(new double[]{0.1 , Double.NaN, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, 2});
      assertEquals(inputVal, reflectObj(inputVal));
     
      inputVal.setIndexNA(new int[]{2});
      assertEquals(inputVal, reflectObj(inputVal));
 
      inputVal.setNames(new String[]{"n1", "nan", "negative_inf", "positive_inf", "n2"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRInteger() throws Exception {
      RInteger inputVal=new RInteger();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setValue(new int[]{1, 0, 100, 2});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setIndexNA(new int[]{1, 3});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setNames(new String[]{"n1", "n2", "n3", "n4"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRChar() throws Exception {
      RChar inputVal=new RChar();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setValue(new String[]{"a", "weird", "b"});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setIndexNA(new int[]{1});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setNames(new String[]{"n1", "n2", "n3"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRComplex() throws Exception {
      RComplex inputVal=new RComplex();
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setReal(new double[]{1, Double.NaN, 3});
      inputVal.setImaginary(new double[]{-1, 0, 1});
      assertEquals(inputVal, reflectObj(inputVal));
      
      inputVal.setIndexNA(new int[]{2});
      assertEquals(inputVal, reflectObj(inputVal));
 
      inputVal.setNames(new String[]{"n1", "n2", "n3"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRList() throws Exception {
      RList inputVal=new RList();
      assertEquals(inputVal, reflectObj(inputVal));

      ArrayList<Object> listData=new ArrayList<Object>();
      RChar eleVal1=new RChar();
      eleVal1.setValue(new String[]{"a", "weird", "b"});
      eleVal1.setIndexNA(new int[]{1});
      eleVal1.setNames(new String[]{"n1", "n2", "n3"});
      RInteger eleVal2=new RInteger();
      eleVal2.setValue(new int[]{1, 0, 100, 2});
      eleVal2.setIndexNA(new int[]{1, 3});
      eleVal2.setNames(new String[]{"n1", "n2", "n3", "n4"});
      listData.add(eleVal1);
      listData.add(eleVal2);
      inputVal.setValue((Object[])listData.toArray(new Object[listData.size()]));
      assertEquals(inputVal, reflectObj(inputVal));

      inputVal.setNames(new String[]{"data", "name"});
      assertEquals(inputVal, reflectObj(inputVal));
    }
    
    @Test
    public void testRArray() throws Exception {
      RArray inputVal=new RArray();
      assertEquals(inputVal, reflectObj(inputVal));

      RInteger arrayData=new RInteger();
      arrayData.setValue(new int[]{1, 2, 3, 4});
      arrayData.setNames(new String[]{"n1", "n2", "n3", "n4"});
      inputVal.setValue(arrayData);
      inputVal.setDim(new int[]{1, 2, 2}); 
      assertEquals(inputVal, reflectObj(inputVal));

      RList dimnames=new RList();
      Object[] dimnameData=new Object[3];
      dimnameData[0]=new RChar(new String[]{"z"}, null, null);
      dimnameData[1]=new RChar(new String[]{"x1", "x2"}, null, null);
      dimnameData[2]=new RChar(new String[]{"y1", "y2"}, null, null);
      dimnames.setValue(dimnameData);
      inputVal.setDimnames(dimnames);
      assertEquals(inputVal, reflectObj(inputVal));
    }

    @Test
    public void testRFactor() throws Exception {
	RFactor inputVal=new RFactor();
      	assertEquals(inputVal, reflectObj(inputVal));
	
	String[] flevels=new String[]{"1", "2", "3", "4", "5"};
	int[] fcode=new int[]{4, 3, 2, 0, 1, -1, 3};
      	inputVal.setLevels(flevels);
      	inputVal.setCode(fcode);
      	assertEquals(inputVal, reflectObj(inputVal));
    }

    @Test
    public void testRDataFrame() throws Exception {
      RDataFrame inputVal=new RDataFrame();
      assertEquals(inputVal, reflectObj(inputVal));
	
      RList listVal=new RList();
      ArrayList<Object> listData=new ArrayList<Object>();
      RChar eleVal1=new RChar();
      eleVal1.setValue(new String[]{"a", "weird", "b"});
      RInteger eleVal2=new RInteger();
      eleVal2.setValue(new int[]{1, 0, 100});
      listData.add(eleVal1);
      listData.add(eleVal2);
      listVal.setValue((Object[])listData.toArray(new Object[listData.size()]));
      listVal.setNames(new String[]{"data", "name"});
      inputVal.setData(listVal);
      inputVal.setRowNames(new String[]{"r1", "r2", "r3"});
      assertEquals(inputVal, reflectObj(inputVal));
    }

    @Test
    public void testREnvironment() throws Exception {
	REnvironment inputVal=new REnvironment();
      	assertEquals(inputVal, reflectObj(inputVal));

	RFactor factorVal=new RFactor();
	String[] flevels=new String[]{"1", "2", "3", "4", "5"};
	int[] fcode=new int[]{4, 3, 2, 0, 1, -1, 3};
      	factorVal.setLevels(flevels);
      	factorVal.setCode(fcode);
      	RInteger integerVal=new RInteger();
      	integerVal.setValue(new int[]{1, 0, 100});
	inputVal.put("my_factor", factorVal);
      	inputVal.put("my_null", null);
      	inputVal.put("my_int", integerVal);
      	assertEquals(inputVal, reflectObj(inputVal));
    }

}

