package org.bioconductor.packages.rservices;

/*
 These test the converters between R and Java.
 Specifically, methods below pass simple data
 from Java to R and get the results back. This
 round-trip should test many of the converters.
*/

import java.util.*;
import org.omegahat.R.Java.ROmegahatInterpreter;
import org.omegahat.R.Java.REvaluator;
import junit.framework.JUnit4TestAdapter;
import org.junit.BeforeClass;
import org.junit.Test;
import org.junit.Ignore;
import static org.junit.Assert.*;

public class SJavaTypeTest extends org.bioconductor.packages.rservices.TypeTestUtil {
    private String sjavaMsg="This test normally fails in pure SJava setting:\n";
    
    /**
     * Used for backward compatibility (IDEs, Ant and JUnit 3.x text runner)
     */
    public static junit.framework.Test suite() {
        return new JUnit4TestAdapter(SJavaTypeTest.class);
    }
    
    @Test
    public void testNull() throws Exception {
      assertNull(reflectObj(null));
    }

    @Test
    public void testDouble() throws Exception {
      Double DoubleVal0=new Double(2.3);
      assertEquals(DoubleVal0, reflectObj(DoubleVal0));
      
      double[] doubleVal= new double[]{2.3}; 
      assertEquals(DoubleVal0, reflectObj(doubleVal));
           
      Double[] DoubleVal= new Double[]{new Double(2.3)};
      assertEquals(DoubleVal0, reflectObj(DoubleVal));

      doubleVal= new double[]{2.3, 
                            Double.NaN, 
                            Double.POSITIVE_INFINITY, 
                            Double.NEGATIVE_INFINITY, 
                            5.7,
                            10};
      double[] res=(double[])reflectObj(doubleVal);
      assertTrue(Arrays.equals(doubleVal, res));
      
      DoubleVal= new Double[]{new Double(2.3), 
                            new Double(Double.NaN), 
                            new Double(Double.POSITIVE_INFINITY),
                            new Double(Double.NEGATIVE_INFINITY),
                            new Double(5.7), 
                            new Double(10)};
      res=(double[])reflectObj(DoubleVal);
      assertTrue(Arrays.equals(doubleVal, res));
   
    }

    @Ignore
    @Test
    public void testInteger() throws Exception {
      Integer IntegerVal0=new Integer(2);
      assertEquals(sjavaMsg, IntegerVal0, reflectObj(IntegerVal0));
      
      int[] intVal= new int[]{2};
      assertEquals(sjavaMsg, IntegerVal0, reflectObj(intVal));
      
      Integer[] IntegerVal=new Integer[]{new Integer(2)};
      assertEquals(sjavaMsg, IntegerVal0, reflectObj(IntegerVal));
      
      intVal= new int[]{2, 3, -1, 0, 5};
      int[] res=(int[])reflectObj(intVal);
      assertTrue(sjavaMsg, Arrays.equals(intVal, res));

      IntegerVal= new Integer[]{new Integer(2), new Integer(3), 
                        new Integer(-1), new Integer(0), new Integer(5)};
      res=(int[])reflectObj(IntegerVal);
      assertTrue(sjavaMsg, Arrays.equals(intVal, res));
    }
    
    @Test
    public void testBoolean() throws Exception {
      Boolean BooleanVal0=new Boolean(true);
      assertEquals(BooleanVal0, reflectObj(BooleanVal0));
      
      boolean[] booleanVal= new boolean[]{true}; 
      assertEquals(BooleanVal0, reflectObj(booleanVal));
           
      Boolean[] BooleanVal= new Boolean[]{Boolean.TRUE};
      assertEquals(BooleanVal0, reflectObj(BooleanVal));

      booleanVal= new boolean[]{true, false};
      boolean[] res=(boolean[])reflectObj(booleanVal);
      assertTrue(Arrays.equals(booleanVal, res));
      
      BooleanVal= new Boolean[]{Boolean.TRUE, Boolean.FALSE};
      res=(boolean[])reflectObj(BooleanVal);
      assertTrue(Arrays.equals(booleanVal, res));
   
    }

    @Test
    public void testString() throws Exception {
      String[] StringVal= new String[]{new String("a")};
      assertEquals(new String("a"), reflectObj(StringVal));
      
      StringVal= new String[]{new String("a"), new String("b")};
      String[] res=(String[])reflectObj(StringVal);
      assertTrue(Arrays.equals(StringVal, res));

      StringVal= new String[]{new String("a"), new String("b"), null};
      String[] expRes = new String[]{new String("a"), new String("b"), new String("")};
      res=(String[])reflectObj(StringVal);
      assertTrue(Arrays.equals(expRes, res));
    }

    @Ignore
    @Test
    public void testEmptyDouble() throws Exception {
      double[] doubleVal=new double[]{};
      Object res=reflectObj(doubleVal);
      assertEquals(sjavaMsg, doubleVal.getClass(), res.getClass());
      assertTrue(sjavaMsg, Arrays.equals(doubleVal, (double[])res));
      
      Double[] DoubleVal=new Double[]{};
      res=reflectObj(DoubleVal);
      assertEquals(sjavaMsg, doubleVal.getClass(), res.getClass());
      assertTrue(sjavaMsg, Arrays.equals(doubleVal, (double[])res));
    }

    @Ignore
    @Test
    public void testEmptyInteger() throws Exception {
      int[] intVal=new int[]{};
      Object res=reflectObj(intVal);
      assertEquals(sjavaMsg, intVal.getClass(), res.getClass());
      assertTrue(sjavaMsg, Arrays.equals(intVal, (int[])res));
      
      Integer[] IntegerVal=new Integer[]{};
      assertEquals(sjavaMsg, intVal.getClass(), res.getClass());
      res=reflectObj(IntegerVal);
      assertTrue(sjavaMsg, Arrays.equals(intVal, (int[])res));
    }

    @Ignore
    @Test
    public void testEmptyBoolean() throws Exception {
      boolean[] booleanVal=new boolean[]{};
      Object res=reflectObj(booleanVal);
      assertEquals(sjavaMsg, booleanVal.getClass(), res.getClass());
      assertTrue(Arrays.equals(booleanVal, (boolean[])res));
      
      Boolean[] BooleanVal=new Boolean[]{};
      res=reflectObj(BooleanVal);
      assertEquals(sjavaMsg, booleanVal.getClass(), res.getClass());
      assertTrue(sjavaMsg, Arrays.equals(booleanVal, (boolean[])res));
    }
}

