\name{RJavaPkgFunctions-class}
\docType{class}
\alias{RJavaPkgFunctions-class}
\alias{show,RJavaPkgFunctions-method}
\title{Java signatures derived from R methods}
\description{
  Class \code{RJavaPkgFunctions} contains elements of Java
  signatures, including the source R package, the corresponding java
  class, and type information about the Java function.

  Create this class using \code{\link{createMap}} or \code{\link{generateFunctionMap}}.
}
\section{Slots}{
  \describe{
    \item{\code{rPackage}}{Character vector of R source packages}
    \item{\code{javaClass}}{Character vector of corresponding Java classes}
    \item{\code{javaTypeInfo}}{List of Java function signatures.}
  }
}
\author{MT Morgan <mtmorgan@fhcrc.org> }
\seealso{\code{\link{generateFunctionMap}}, \code{\link{createMap}}}
\examples{
library(RWebServices)

oneWayAnova <- function( response, predictor ) {
    if ( is.character( predictor )) 
        return( oneWayAnova( response, as.factor( predictor )))
    formula <- as.formula( substitute( response ~ predictor ))
    result <- lm( formula )
    anova( result )
}

typeInfo(oneWayAnova) <-
  SimultaneousTypeSpecification(
    TypedSignature(
      response = "numeric",
      predictor = "factor"),
    TypedSignature(
      response = "numeric",
      predictor = "character"),
    returnType = "anova" )

\dontrun{
res <- createMap("oneWayAnova", outputDirectory=tempdir(),
typeMode="robject", verbose=TRUE)
}
}
\keyword{classes}
